#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from fenrirscreenreader.core import debug
from fenrirscreenreader.core.brailleDriver import brailleDriver

class driver(brailleDriver):
    """BRLAPI implementation of the braille driver interface.
    Supports all braille displays compatible with BRLTTY."""
    
    def __init__(self):
        """Initialize the BRLAPI driver."""
        brailleDriver.__init__(self)
        self._brl = None
        self._last_written_text = ""

    def initialize(self, environment):
        """Initialize the braille driver with BRLAPI connection."""
        self.env = environment
        try:
            import brlapi
            self._brl = brlapi.Connection()
            self._deviceSize = self._brl.displaySize
            # Accept all key types from any braille display
            self._brl.acceptKeys(brlapi.rangeType_all, [0, brlapi.KEY_MAX])
            self._current_cursor_pos = 0
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut(str(e), debug.debugLevel.ERROR)
            return
        self._isInitialized = True

    def getDeviceSize(self):
        """Get the size of the connected braille display."""
        if not self._isInitialized or not self._deviceSize:
            return (0, 0)
        return self._deviceSize

    def writeText(self, text):
        """Write text to the braille display."""
        if not self._isInitialized:
            return
        try:
            self._last_written_text = text
            # Handle unicode properly for international braille
            if isinstance(text, str):
                self._brl.writeText(text)
            else:
                self._brl.writeText(text.decode('utf-8', 'replace'))
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.writeText ' + str(e), debug.debugLevel.ERROR)

    def connectDevice(self):
        """Establish connection with BRLAPI."""
        try:
            import brlapi
            self._brl = brlapi.Connection()
            # Accept all key types from the connected display
            self._brl.acceptKeys(brlapi.rangeType_all, [0, brlapi.KEY_MAX])
            return True
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.connectDevice ' + str(e), debug.debugLevel.ERROR)
            return False

    def enterScreen(self, screen):
        """Enter a specific TTY mode."""
        if not self._isInitialized:
            return
        try:
            self._brl.enterTtyMode(int(screen))
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.enterScreen ' + str(e), debug.debugLevel.ERROR)

    def leaveScreen(self):
        """Leave the current TTY mode."""
        if not self._isInitialized:
            return
        try:
            self._brl.leaveTtyMode()
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.leaveScreen ' + str(e), debug.debugLevel.ERROR)

    def setCursor(self, position):
        """Set the cursor position on the braille display."""
        if not self._isInitialized:
            return
        try:
            max_pos = self.getDeviceSize()[0] - 1
            self._current_cursor_pos = max(0, min(position, max_pos))
            # Use BRLAPI's cursor command which works across different displays
            self._brl.writeDots(1 << self._current_cursor_pos)
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.setCursor ' + str(e), debug.debugLevel.ERROR)

    def getCursorPosition(self):
        """Get the current cursor position."""
        if not self._isInitialized:
            return 0
        return self._current_cursor_pos

    def clear(self):
        """Clear the braille display."""
        if not self._isInitialized:
            return
        try:
            self._brl.writeText('')
            self._last_written_text = ""
            self._current_cursor_pos = 0
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.clear ' + str(e), debug.debugLevel.ERROR)

    def flush(self):
        """Force an update of the physical display."""
        if not self._isInitialized:
            return
        try:
            if self._last_written_text:
                self._brl.writeText(self._last_written_text)
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.flush ' + str(e), debug.debugLevel.ERROR)

    def handleKeyInput(self, key):
        """Handle input from any BRLTTY-compatible braille display.
        Returns the key event for processing by Fenrir's input system."""
        if not self._isInitialized:
            return False
        try:
            key_code = self._brl.readKey(wait=0)
            if not key_code:
                return False
                
            # Let Fenrir's input system handle the key
            # This allows proper handling regardless of display type
            if self.env['runtime']['debug'].debugLevel >= debug.debugLevel.INFO:
                self.env['runtime']['debug'].writeDebugOut(
                    'BRAILLE.keyPressed: ' + str(key_code), 
                    debug.debugLevel.INFO
                )
            return True
        except brlapi.ConnectionError:
            self.env['runtime']['debug'].writeDebugOut(
                'BRAILLE: Connection lost. Attempting to reconnect...',
                debug.debugLevel.ERROR
            )
            self.connectDevice()
            return False
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut(
                'BRAILLE.handleKeyInput ' + str(e),
                debug.debugLevel.ERROR
            )
            return False

    def shutdown(self):
        """Shut down the BRLAPI driver."""
        if not self._isInitialized:
            return
        try:
            self.clear()
            self.leaveScreen()
            if self._brl:
                self._brl.closeConnection()
            self._brl = None
            self._isInitialized = False
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut('BRAILLE.shutdown ' + str(e), debug.debugLevel.ERROR)
