#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributors.

from abc import ABC, abstractmethod
from fenrirscreenreader.core import debug

class brailleDriver(ABC):
    """Base class for Fenrir braille display drivers.
    
    This abstract base class defines the interface that all braille drivers must implement.
    It provides basic initialization state tracking and defines the required methods
    for interacting with braille displays.
    """
    
    def __init__(self):
        """Initialize the driver with default state."""
        self._isInitialized = False
        self.deviceSize = None
        self.env = None
        self._current_cursor_pos = 0
        self._display_content = ""
    
    @abstractmethod
    def initialize(self, environment):
        """Initialize the braille driver with the given environment.
        
        Args:
            environment (dict): The Fenrir environment dictionary containing runtime settings
                              and helper objects.
        """
        self.env = environment
        self._isInitialized = True

    @abstractmethod
    def getDeviceSize(self):
        """Get the size of the connected braille display.
        
        Returns:
            tuple: A (columns, rows) tuple indicating the display dimensions.
                  Returns (0, 0) if no display is connected or not initialized.
        """
        if not self._isInitialized:
            return (0, 0)
        return (0, 0)

    @abstractmethod
    def writeText(self, text):
        """Write text to the braille display.
        
        Args:
            text (str): The text to be written to the display.
        """
        if not self._isInitialized:
            return

    @abstractmethod
    def connectDevice(self):
        """Establish connection with the braille display.
        
        This method should handle the initial connection setup with the display.
        It should be called before any other display operations.
        """
        pass

    @abstractmethod
    def enterScreen(self, screen):
        """Enter a specific screen/TTY mode.
        
        Args:
            screen (int): The screen/TTY number to enter.
        """
        if not self._isInitialized:
            return

    @abstractmethod
    def leaveScreen(self):
        """Leave the current screen/TTY mode.
        
        This method should clean up any screen-specific state and
        prepare for potential screen switches.
        """
        if not self._isInitialized:
            return

    @abstractmethod
    def setCursor(self, position):
        """Set the cursor position on the braille display.
        
        Args:
            position (int): The position where the cursor should be placed.
        """
        if not self._isInitialized:
            return
        self._current_cursor_pos = max(0, min(position, self.getDeviceSize()[0] - 1))

    @abstractmethod
    def getCursorPosition(self):
        """Get the current cursor position on the braille display.
        
        Returns:
            int: The current cursor position.
        """
        if not self._isInitialized:
            return 0
        return self._current_cursor_pos

    @abstractmethod
    def clear(self):
        """Clear the braille display."""
        if not self._isInitialized:
            return
        self._display_content = ""
        self._current_cursor_pos = 0

    @abstractmethod
    def flush(self):
        """Force an update of the physical display with current content."""
        if not self._isInitialized:
            return

    def shutdown(self):
        """Shut down the braille driver and clean up resources.
        
        This method ensures proper cleanup of resources and
        disconnection from the braille display.
        """
        if not self._isInitialized:
            return
        self.clear()
        self.leaveScreen()
        self._isInitialized = False

    @property
    def is_initialized(self):
        """Check if the driver is initialized.
        
        Returns:
            bool: True if the driver is initialized, False otherwise.
        """
        return self._isInitialized

    @abstractmethod
    def handleKeyInput(self, key):
        """Handle input from the braille display's keys.
        
        Args:
            key: The key event from the braille display.
        Returns:
            bool: True if the key was handled, False otherwise.
        """
        if not self._isInitialized:
            return False
        return False
